#include "btransforms.vsh"
#include "blightmats.vsh"

float4x4 mViewProj;  // View * Projection transformation

#if D_Tex >= 4
float4x4 mShadowView;
#endif

struct VS_INPUT
{

#if D_PosDimension == 3
    float3  Pos             : POSITION0;
#elif D_PosDimension == 4
    float4  Pos             : POSITION0;
#endif

#ifdef D_BlendWeights
    float4  BlendWeights    : BLENDWEIGHT0;
    float4  BlendIndices    : BLENDINDICES0;
#endif

#ifdef D_Normal
    float3  Normal          : NORMAL0;
#endif

#ifdef D_Tangent
	float3  Tangent         : TANGENT0;
#endif

#ifdef D_Color0
	float4  Diffuse			: COLOR0;
#endif

#ifdef D_Color1
	float4  Specular		: COLOR1;
#endif

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif



};

struct VS_OUTPUT
{
    float4  Pos     : POSITION;
    float4  Diffuse : COLOR0;

	float4  Specular: COLOR1;
	

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif

	float3  Eye             : TEXCOORD1;

#ifdef D_Normal
	float3  Normal          : TEXCOORD2;


#ifdef D_Tangent
	float3  Tangent         : TEXCOORD3;

#if D_Tex >= 4
	float3  ShadowPos       : TEXCOORD4;
#endif

#define Normal_Tangent
#else

#if D_Tex >= 4
	float3  ShadowPos       : TEXCOORD3;
#endif

#endif

#else

#if D_Tex >= 4
	float3  ShadowPos       : TEXCOORD2;
#endif

#endif



};


VS_OUTPUT VS_Main(VS_INPUT i)
{
	VS_OUTPUT o;// = (VS_OUTPUT)0;

	VS_TRANS_INPUT i2;
	VS_TRANS_OUTPUT o2;

	i2.Pos = i.Pos;
	
#ifdef D_BlendWeights
	i2.BlendWeights = i.BlendWeights;
	i2.BlendIndices = i.BlendIndices;
#endif

#ifdef D_Normal
    i2.Normal = i.Normal;
#endif

#ifdef D_Tangent
    i2.Tangent = i.Tangent;
#endif

	o2 = Transform(i2);


	o.Pos = mul(float4(o2.Pos.xyz, 1.0f), mViewProj);


	float3 E = o2.Pos.xyz - eyepos;
	o.Eye = E;

#ifdef D_Normal
/*
    o.Diffuse = float4(Lambert(o2.Normal) * diffusematerial 
							+ ambientmaterial.xyz
						, diffusematerial.w);

	o.Specular = Phong(o2.Normal, o2.Pos.xyz) * specularmaterial;
*/
	o.Diffuse = float4(1.0f, 1.0f, 1.0f, 1.0f);
	o.Specular = float4(0.0f, 0.0f, 0.0f, 0.0f);

	o.Normal = o2.Normal;

	//o.Eye.x = dot(E, o2.Tangent);
	//o.Eye.y = dot(E, B);
	//o.Eye.z = dot(E, o2.Normal);
	

#ifdef D_Tangent
	o.Tangent = o2.Tangent;

	//o.Tangent = float3(0.0f, 0.0f, 1.0f);

#endif

/*
#if D_Tex >= 3
#ifndef Normal_Tangent
	o.EnvMapCood = PhongRef(o2.Normal, o2.Pos.xyz);
#else
	//float3 B = cross(o2.Normal, o2.Tangent);



	//float3 L = lightdir;

	//o.Light.x = dot(L, o2.Tangent);
	//o.Light.y = dot(L, B);
	//o.Light.z = dot(L, o2.Normal);
	
	//o.Light = L;


#endif
#endif
*/

	//o.Specular = specularmaterial;

#else
	o.Diffuse = float4(1.0f, 1.0f, 1.0f, 1.0f);
	o.Specular = float4(0.0f, 0.0f, 0.0f, 0.0f);
#endif

#ifdef D_Color1
	o.Specular = i.Specular;
#endif

#ifdef D_Color0
	o.Diffuse = i.Diffuse * o.Diffuse;
#endif

//#ifdef D_BlendWeights
//	o.Diffuse.xyz = i.BlendIndices.xyz;
//	o.Diffuse.xyz = i.BlendWeights;
//#endif

//#ifdef Normal_Tangent
//	o.Diffuse.xyz = o.Normal;
//#endif

#ifdef D_Color1
	//o.Specular = i.Specular * o.Specular;
	//o.Specular = i.Specular;
#endif


#if D_Texcoods >= 1
	o.Tex0 = i.Tex0;
#endif

#if D_Tex >= 4
	//o.ShadowPos = mul(float4(o2.Pos.xyz, 1.0f), mShadowView).xyz
	//			 + float3(1.001f, -1.001f, 0.0f)
	//			;
	//o.ShadowPos.xy *= float2(0.5f, -0.5f);
	//o.ShadowPos.z = 1.0f - o.ShadowPos.z;

	float4 tshadowpos;
	tshadowpos = mul(float4(o2.Pos.xyz, 1.0f), mShadowView)
				 + float4(1.001f, -1.001f, 0.0f, 0.0f)
				;
	tshadowpos /= tshadowpos.w;
	//tshadowpos.x -= 0.5f;
	//tshadowpos.y += 0.5f;

	o.ShadowPos = tshadowpos.xyz;

	o.ShadowPos.xy *= float2(0.5f, -0.5f);
	o.ShadowPos.z = 1.0f - o.ShadowPos.z;
#endif


	return o;
}

