#include "btransforms.vsh"
#include "blightmats.vsh"

float4x4 mViewProj;  // View * Projection transformation



struct VS_INPUT
{

#if D_PosDimension == 3
    float3  Pos             : POSITION;
#elif D_PosDimension == 4
    float4  Pos             : POSITION;
#endif

#ifdef D_BlendWeights
    float3  BlendWeights    : BLENDWEIGHT;
    float4  BlendIndices    : BLENDINDICES;
#endif

#ifdef D_Normal
    float3  Normal          : NORMAL;
#endif

#ifdef D_Color0
	float4  Diffuse			: COLOR0;
#endif

#ifdef D_Color1
	float4  Specular		: COLOR1;
#endif

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif

#if D_Texcoods >= 2
#if D_TexDim1 == 2
    float2  Tex1            : TEXCOORD1;
#elif D_TexDim1 == 3
    float3  Tex1            : TEXCOORD1;
#elif D_TexDim1 == 4
    float4  Tex1            : TEXCOORD1;
#endif
#endif

#if D_Texcoods >= 3
#if D_TexDim2 == 2
    float2  Tex2            : TEXCOORD2;
#elif D_TexDim2 == 3
    float3  Tex2            : TEXCOORD2;
#elif D_TexDim2 == 4
    float4  Tex2            : TEXCOORD2;
#endif
#endif

#if D_Texcoods >= 4
#if D_TexDim3 == 2
    float2  Tex3            : TEXCOORD3;
#elif D_TexDim3 == 3
    float3  Tex3            : TEXCOORD3;
#elif D_TexDim3 == 4
    float4  Tex3            : TEXCOORD3;
#endif
#endif

#if D_Texcoods >= 5
#if D_TexDim4 == 2
    float2  Tex4            : TEXCOORD4;
#elif D_TexDim4 == 3
    float3  Tex4            : TEXCOORD4;
#elif D_TexDim4 == 4
    float4  Tex4            : TEXCOORD4;
#endif
#endif

#if D_Texcoods >= 6
#if D_TexDim5 == 2
    float2  Tex5            : TEXCOORD5;
#elif D_TexDim4 == 3
    float3  Tex5            : TEXCOORD5;
#elif D_TexDim4 == 4
    float4  Tex5            : TEXCOORD5;
#endif
#endif

#if D_Texcoods >= 7
#if D_TexDim6 == 2
    float2  Tex6            : TEXCOORD6;
#elif D_TexDim4 == 3
    float3  Tex6            : TEXCOORD6;
#elif D_TexDim4 == 4
    float4  Tex6            : TEXCOORD6;
#endif
#endif

#if D_Texcoods >= 8
#if D_TexDim7 == 2
    float2  Tex7            : TEXCOORD7;
#elif D_TexDim4 == 3
    float3  Tex7            : TEXCOORD7;
#elif D_TexDim4 == 4
    float4  Tex7            : TEXCOORD7;
#endif
#endif


};

struct VS_OUTPUT
{
    float4  Pos     : POSITION;
    float4  Diffuse : COLOR0;

	float4  Specular: COLOR1;

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif

#if D_Texcoods >= 2
#if D_TexDim1 == 2
    float2  Tex1            : TEXCOORD1;
#elif D_TexDim1 == 3
    float3  Tex1            : TEXCOORD1;
#elif D_TexDim1 == 4
    float4  Tex1            : TEXCOORD1;
#endif
#endif

#if D_Texcoods >= 3
#if D_TexDim2 == 2
    float2  Tex2            : TEXCOORD2;
#elif D_TexDim2 == 3
    float3  Tex2            : TEXCOORD2;
#elif D_TexDim2 == 4
    float4  Tex2            : TEXCOORD2;
#endif
#endif

#if D_Texcoods >= 4
#if D_TexDim3 == 2
    float2  Tex3            : TEXCOORD3;
#elif D_TexDim3 == 3
    float3  Tex3            : TEXCOORD3;
#elif D_TexDim3 == 4
    float4  Tex3            : TEXCOORD3;
#endif
#endif

#if D_Texcoods >= 5
#if D_TexDim4 == 2
    float2  Tex4            : TEXCOORD4;
#elif D_TexDim4 == 3
    float3  Tex4            : TEXCOORD4;
#elif D_TexDim4 == 4
    float4  Tex4            : TEXCOORD4;
#endif
#endif

#if D_Texcoods >= 6
#if D_TexDim5 == 2
    float2  Tex5            : TEXCOORD5;
#elif D_TexDim4 == 3
    float3  Tex5            : TEXCOORD5;
#elif D_TexDim4 == 4
    float4  Tex5            : TEXCOORD5;
#endif
#endif

#if D_Texcoods >= 7
#if D_TexDim6 == 2
    float2  Tex6            : TEXCOORD6;
#elif D_TexDim4 == 3
    float3  Tex6            : TEXCOORD6;
#elif D_TexDim4 == 4
    float4  Tex6            : TEXCOORD6;
#endif
#endif

#if D_Texcoods >= 8
#if D_TexDim7 == 2
    float2  Tex7            : TEXCOORD7;
#elif D_TexDim4 == 3
    float3  Tex7            : TEXCOORD7;
#elif D_TexDim4 == 4
    float4  Tex7            : TEXCOORD7;
#endif
#endif

};


VS_OUTPUT VS_Main(VS_INPUT i)
{
	VS_OUTPUT o = (VS_OUTPUT)0;

	VS_TRANS_INPUT i2;
	VS_TRANS_OUTPUT o2;

	i2.Pos = i.Pos;
	
#ifdef D_BlendWeights
	i2.BlendWeights = i.BlendWeights;
	i2.BlendIndices = i.BlendIndices;
#endif

#ifdef D_Normal
    i2.Normal = i.Normal;
#endif

	o2 = Transform(i2);


	o.Pos = mul(float4(o2.Pos.xyz, 1.0f), mViewProj);


#ifdef D_Normal
    o.Diffuse = float4(Lambert(o2.Normal) * diffusematerial 
							+ ambientmaterial.xyz
						, diffusematerial.w);

	o.Specular = Phong(o2.Normal, o2.Pos.xyz) * specularmaterial;
#else
	o.Diffuse = float4(1.0f, 1.0f, 1.0f, 1.0f);
	o.Specular = float4(0.0f, 0.0f, 0.0f, 0.0f);
#endif

#ifdef D_Color0
	o.Diffuse = i.Diffuse * o.Diffuse;
#endif

//#ifdef D_BlendWeights
//	o.Diffuse.xyz = i.BlendIndices.xyz;
//	o.Diffuse.xyz = i.BlendWeights;
//#endif


#ifdef D_Color1
	o.Specular = i.Specular * o.Specular;
#endif


#if D_Texcoods >= 1
	o.Tex0 = i.Tex0;
#endif

#if D_Texcoods >= 2
	o.Tex1 = i.Tex1;
#endif

#if D_Texcoods >= 3
	o.Tex2 = i.Tex2;
#endif

#if D_Texcoods >= 4
	o.Tex3 = i.Tex3;
#endif

#if D_Texcoods >= 5
	o.Tex4 = i.Tex4;
#endif

#if D_Texcoods >= 6
	o.Tex5 = i.Tex5;
#endif

#if D_Texcoods >= 7
	o.Tex6 = i.Tex6;
#endif

#if D_Texcoods >= 8
	o.Tex7 = i.Tex7;
#endif

	return o;
}

