#include "btransforms.vsh"
#include "blightmats.vsh"

float4x4 mViewProj;  // View * Projection transformation
float4x4 mShadowView;


struct VS_INPUT
{

#if D_PosDimension == 3
    float3  Pos             : POSITION0;
#elif D_PosDimension == 4
    float4  Pos             : POSITION0;
#endif

#ifdef D_BlendWeights
    float4  BlendWeights    : BLENDWEIGHT0;
    float4  BlendIndices    : BLENDINDICES0;
#endif

#ifdef D_Normal
    float3  Normal          : NORMAL0;
#endif

#ifdef D_Tangent
	float3  Tangent         : TANGENT0;
#endif

#ifdef D_Color0
	float4  Diffuse			: COLOR0;
#endif

#ifdef D_Color1
	float4  Specular		: COLOR1;
#endif

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif



};

struct VS_OUTPUT
{
    float4  Pos     : POSITION;
    float4  Depth : COLOR0;

#if D_Texcoods >= 1
#if D_TexDim0 == 2
    float2  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 3
    float3  Tex0            : TEXCOORD0;
#elif D_TexDim0 == 4
    float4  Tex0            : TEXCOORD0;
#endif
#endif
};


VS_OUTPUT VS_Main(VS_INPUT i)
{
	VS_OUTPUT o;// = (VS_OUTPUT)0;

	VS_TRANS_INPUT i2;
	VS_TRANS_OUTPUT o2;

	i2.Pos = i.Pos;
	
#ifdef D_BlendWeights
	i2.BlendWeights = i.BlendWeights;
	i2.BlendIndices = i.BlendIndices;
#endif

#ifdef D_Normal
    i2.Normal = i.Normal;
#endif

#ifdef D_Tangent
    i2.Tangent = i.Tangent;
#endif

	o2 = Transform(i2);

	o.Pos = mul(float4(o2.Pos.xyz, 1.0f), mShadowView);
	//o.Pos = mul(float4(o2.Pos.xyz, 1.0f), mViewProj);

	float alpha = 1.0f;

#ifdef D_Color0
	alpha = i.Diffuse;
#endif

	float idepth;
	idepth = 1.0f - o.Pos.z / o.Pos.w;

	o.Depth = float4(idepth, idepth, idepth, alpha);

	o.Tex0 = i.Tex0;

	return o;
}

