#ifndef btransformh
#define btransformh


#if D_Matsize == 1

float4x3 mWorld;

#else

float4x3 mWorld[D_Matsize];

#endif

struct VS_TRANS_INPUT
{
#if D_PosDimension == 3
    float3  Pos;
#elif D_PosDimension == 4
    float4  Pos;
#endif

#ifdef D_BlendWeights
    float4  BlendWeights;
    float4  BlendIndices;
#endif

#ifdef D_Normal
    float3  Normal;
#endif

#ifdef D_Tangent
	float3  Tangent;
#endif
};

struct VS_TRANS_OUTPUT
{
    float4 Pos;

#ifdef D_Normal
    float3 Normal;
#endif

#ifdef D_Tangent
	float3  Tangent;
#endif
};

VS_TRANS_OUTPUT Transform(VS_TRANS_INPUT i)
{
	VS_TRANS_OUTPUT o = (VS_TRANS_OUTPUT)0;


#if D_Matsize == 1

#if D_PosDimension == 3
	o.Pos.xyz = mul(float4(i.Pos, 1.0f), mWorld);
#elif D_PosDimension == 4
	o.Pos.xyz = mul(i.Pos, mWorld);
#endif
	o.Pos.w = 1.0f;

#ifdef D_Normal
	o.Normal = mul(i.Normal, mWorld);
#endif

#ifdef D_Tangent
	o.Tangent = mul(i.Tangent, mWorld);
#endif

#else

//	float lw = 1.0f;
	float w;
	float bw[ 4 ] = (float[ 4 ]) i.BlendWeights;
    int ia[ 4 ] = (int[ 4 ]) D3DCOLORtoUBYTE4( i.BlendIndices );

#ifdef D_Normal
	float3 nrml = float3(0.0f, 0.0f, 0.0f);
#endif

#ifdef D_Tangent
	float3 tng = float3(0.0f, 0.0f, 0.0f);
#endif

	for(int j = 0 ; j < 4 ; j++)
	{
		w = bw[j];
//		lw -= w;

#if D_PosDimension == 3
		o.Pos +=  mul(float4(i.Pos, 1.0f), mWorld[ia[j]]) * w;
#elif D_PosDimension == 4
		o.Pos.xyz +=  mul(i.Pos, mWorld[ia[j]]) * w;
#endif

#ifdef D_Normal
		nrml +=  mul(i.Normal, mWorld[ia[j]]) * w;
#endif

#ifdef D_Tangent
		tng +=  mul(i.Tangent, mWorld[ia[j]]) * w;
#endif
	}

#if D_PosDimension == 3
//	o.Pos +=  mul(float4(i.Pos, 1.0f), mWorld[ia[3]]) * lw;
#elif D_PosDimension == 4
//	o.Pos.xyz +=  mul(i.Pos, mWorld[ia[3]]) * lw;
	o.Pos.w = 1.0f;
#endif

#ifdef D_Normal
//	nrml +=  mul(i.Normal, mWorld[ia[3]]) * lw;
	o.Normal = normalize(nrml);
#endif

#ifdef D_Tangent
//	tng +=  mul(i.Tangent, mWorld[ia[3]]) * lw;

	o.Tangent = tng;

/*
	if(length(tng) < 0.1f)
	{
		o.Tangent = float3(0.0f, 0.0f ,1.0f);
		
	}
	else
	{
		o.Tangent = normalize(tng);
	}
*/

#endif


#endif



	return o;
}

#endif
